package pragmateek.gson;

import static org.junit.Assert.*;

import java.util.*;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import com.google.gson.*;
import com.google.gson.typeadapters.RuntimeTypeAdapterFactory;

public class Tests {

	@Test
	public void canSerializeATree()
	{
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.MILLISECOND, 0);
		Date time = calendar.getTime();
		
		Folder root = new Folder
		("/",
				new Folder("tmp",
						new File("cache.tmp", time, true)),
				new Folder("opt",
						new Folder("Chrome",
								new File("chrome", 123456))),
				new Folder("home",
						new Folder("john"),
						new Folder("kate"))				
		);
		
		Gson gson = new Gson();
		String json = gson.toJson(root);
		
		Folder outputRoot = gson.fromJson(json, Folder.class);
		
		assertEquals("/", outputRoot.getName());
		
		assertEquals(3, outputRoot.getFolders().length);
		assertEquals("tmp", outputRoot.getFolders()[0].getName());
		assertEquals("opt", outputRoot.getFolders()[1].getName());
		assertEquals("home", outputRoot.getFolders()[2].getName());
		
		assertEquals(1, outputRoot.getFolders()[0].getFiles().length);
		assertEquals("cache.tmp", outputRoot.getFolders()[0].getFiles()[0].getName());
		assertTrue(outputRoot.getFolders()[0].getFiles()[0].getCreationDate().equals(time));
		assertTrue(outputRoot.getFolders()[0].getFiles()[0].isHidden());
		
		assertEquals("chrome", outputRoot.getFolders()[1].getFolders()[0].getFiles()[0].getName());
		assertEquals(123456, outputRoot.getFolders()[1].getFolders()[0].getFiles()[0].getSize());
		
		assertEquals("john", outputRoot.getFolders()[2].getFolders()[0].getName());		
		assertEquals("kate", outputRoot.getFolders()[2].getFolders()[1].getName());
	}

	@Test
	public void canCustomizeTheNames()
	{
		int timestamp = (int)new Date().getTime() / 1000;
		
		Mail mail = new Mail(timestamp);
		
		Gson gson = new Gson();
		
		String json = gson.toJson(mail);
		
		assert(json.contains("\"time\":"));
		
		Mail outputMail = gson.fromJson(json, Mail.class);
		
		assertEquals(timestamp, outputMail.getTimestamp());
	}
	
	@Rule
	public ExpectedException expectedException = ExpectedException.none();
	
	@Test
	public void cannotMapFromIntegerBooleansToTrueBooleans()
	{
		Gson gson = new Gson();
		
    	expectedException.expect(JsonSyntaxException.class);
    	
		File file = gson.fromJson("{hidden: 1}", File.class); 
	}
	
	@Test
	public void canMapFromIntegerBooleansToTrueBooleans()
	{
		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapter(Boolean.class, new BooleanTypeAdapter());
		Gson gson = builder.create();
    	
		File file = gson.fromJson("{hidden: 1}", File.class); 
	}
	
	@Test
	public void cannotMapFromTrueBooleansToIntegerBooleans()
	{
		Gson gson = new Gson();
		
    	expectedException.expect(JsonSyntaxException.class);
    	
		ThirdPartyFile file = gson.fromJson("{hidden: true}", ThirdPartyFile.class); 
	}
	
	@Test
	public void canMapToAndFromThirdParty()
	{
		File file = new File("test.txt");
		file.setHidden(true);
		
		assertTrue(file.isHidden());
		
		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapter(Boolean.class, new BooleanTypeAdapter());
		Gson gson = builder.create();
		
		// We generate some JSON for the third-party API...
		String json = gson.toJson(file);
		
		// we send the JSON to the API...

		// it parses the file in its own representation...
		ThirdPartyFile otherFile = gson.fromJson(json, ThirdPartyFile.class);
		
		assertEquals(1, otherFile.isHidden());
		
		// then it returns us the same file as its JSON representation...
		String otherJson = gson.toJson(otherFile);
		
		// we finally try to read it and ensure all is fine.
		File outFile = gson.fromJson(otherJson, File.class);
		
		assertTrue(outFile.isHidden());
	}
	
	@Test
	public void canMapStatusCodes()
	{		
		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapter(Status.class, new StatusTypeAdapter());
		Gson gson = builder.create();
		
		Response responseOK = gson.fromJson("{status:0}", Response.class);
		Response responseKO = gson.fromJson("{status:1}", Response.class);
		
		assertEquals(Status.OK, responseOK.getStatus());
		assertEquals(Status.KO, responseKO.getStatus());
	}
	
	@Test
	public void canCustomizeTheDateFormatting()
	{
		Calendar calendar = Calendar.getInstance();
		calendar.set(2013, 05, 8, 18, 05, 23);
		calendar.set(Calendar.MILLISECOND, 0);
		Date time = calendar.getTime();
		
		GsonBuilder builder = new GsonBuilder();
		builder.setDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		Gson gson = builder.create();
		
		String json = gson.toJson(time); 
		
		assertEquals("\"2013-06-08T18:05:23\"", json);
		
		Date outTime = gson.fromJson(json, Date.class);
		
		assertTrue(outTime.equals(time));
		
		return;
	}

	@Test
	public void canWriteAndReadCurrencyPairs()
	{
		CurrencyPair EURUSD = new CurrencyPair("EUR", "USD");
		
		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapter(CurrencyPair.class, new CurrencyPairTypeAdapter());
		Gson gson = builder.create();
		
		String json = gson.toJson(EURUSD);
		
		assertEquals("\"EUR/USD\"", json);
		
		CurrencyPair outEURUSD = gson.fromJson(json, CurrencyPair.class);
		
		assertEquals(EURUSD.getBaseCurrency(), outEURUSD.getBaseCurrency());
		assertEquals(EURUSD.getCounterCurrency(), outEURUSD.getCounterCurrency());
	}
	
	@Test
	public void cannotPreserveTypeInformation()
	{
		Folder2 folder = new Folder2("/", new Folder2("tmp"), new File("test.txt"));
		
		assertTrue(folder.getItems()[0] instanceof Folder2);
		assertTrue(folder.getItems()[1] instanceof File);
		
		Gson gson = new Gson();
		
		String json = gson.toJson(folder);
		
		Folder2 outFolder = gson.fromJson(json, Folder2.class);
		
		assertFalse(outFolder.getItems()[0] instanceof Folder2);
		assertFalse(outFolder.getItems()[1] instanceof File);
	}
	
	@Test
	public void canPreserveTypeInformationForASimpleTree()
	{
		Folder2 folder = new Folder2("/", new Folder2("tmp"), new File("test.txt", 123));
		
		assertTrue(folder.getItems()[0] instanceof Folder2);
		assertTrue(folder.getItems()[1] instanceof File);
		
		RuntimeTypeAdapterFactory<FileSystemItem> factory = RuntimeTypeAdapterFactory.of(FileSystemItem.class, "$type")
				 .registerSubtype(File.class)
				 .registerSubtype(Folder2.class);

		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapterFactory(factory);
		Gson gson = builder.create();
		
		String json = gson.toJson(folder);
		
		Folder2 outFolder = gson.fromJson(json, Folder2.class);
		
		assertTrue(outFolder.getName().equals("/"));
		
		assertTrue(outFolder.getItems()[0] instanceof Folder2);
		assertTrue(outFolder.getItems()[1] instanceof File);
		
		assertTrue(outFolder.getItems()[0].getName().equals("tmp"));
		assertTrue(outFolder.getItems()[1].getName().equals("test.txt"));
		assertEquals(123, ((File)outFolder.getItems()[1]).getSize());
	}
	
	@Test
	public void canPreserveTypeInformation()
	{
		RuntimeTypeAdapterFactory<FileSystemItem> factory = RuntimeTypeAdapterFactory.of(FileSystemItem.class, "$type")
																					 .registerSubtype(File.class)
																					 .registerSubtype(Folder2.class);
		
		GsonBuilder builder = new GsonBuilder();
		builder.registerTypeAdapterFactory(factory);
		Gson gson = builder.create();
		
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.MILLISECOND, 0);
		Date time = calendar.getTime();
		
		Folder2 root = new Folder2
				("/",
						new Folder2("tmp",
								new File("cache.tmp", time, true)),
						new Folder2("opt",
								new Folder2("Chrome",
										new File("chrome", 123456))),
						new Folder2("home",
								new Folder2("john"),
								new Folder2("kate"))				
				);
		
		String json = gson.toJson(root);
		
		Folder2 outputRoot = gson.fromJson(json, Folder2.class);
		
		return;
	}
}
